use btlib::Result;
use btproto::protocol;
use btrun::{ActorName, CallMsg};
use btsector::FileId;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct Open {
    id: FileId,
}

impl CallMsg for Open {
    type Reply = ActorName;
}
protocol! {
    named FsProtocol;
    let server = [Listening];
    let client = [Client];
    let file = [FileInit, Opened];
    let file_handle = [FileHandle];

    Client -> Client, >service(Listening)!Query;
    Listening?Query -> Listening, >Client!Query::Reply;
    Client?Query::Reply -> Client;

    Client -> Client, >service(Listening)!Open;
    Listening?Open -> Listening, FileInit, >Client!Open::Reply[Opened], FileInit!Open;
    Client?Open::Reply[Opened] -> Client, FileHandle[Opened];

    FileInit?Open -> Opened;

    FileHandle[Opened] -> FileHandle[Opened], >Opened!FileOp;
    Opened?FileOp -> Opened, >FileHandle!FileOp::Reply;
    FileHandle?FileOp::Reply -> FileHandle;

    FileHandle[Opened] -> End, >Opened!Close;
    Opened?Close -> End;
}

#[derive(Serialize, Deserialize)]
pub struct Query;

impl CallMsg for Query {
    type Reply = ();
}

#[derive(Serialize, Deserialize)]
pub struct FileOp;

impl CallMsg for FileOp {
    type Reply = ();
}

#[derive(Serialize, Deserialize)]
pub struct Close;
